@@ -1,7 +1,6 @@
 #!/usr/bin/perl
 
-# for my dev env
-use lib qw(/projects/lib);
+use lib qw(../blib/lib);
 
 use WWW::GMail;
 use strict;
@@ -18,6 +17,8 @@ my $g = WWW::GMail->new(
 #	debug => 1,
 );
 
+$0 = (split(/ /,$0,1))[0]; # so user/pass don't show up in ps
+
 my $ret = $g->login();
 if ($ret == -1) {
 	print "password incorrect\n";
